﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 10 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class ChangeAnnualFee
    Private ActiveCustomerID As Long

    Public Function PromptUser(ByVal whichCustomer As Long) As Boolean
        ' ----- Prompt the user to change a customer's annual fee.
        ActiveCustomerID = whichCustomer
        If (Me.ShowDialog() = DialogResult.OK) Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub ChangeAnnualFee_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim commandWrapper As SqlCommand
        Dim customerReader As SqlDataReader = Nothing

        ' ----- Retrieve the customer data.
        Using linkToDB As SqlConnection = OpenConnection()
            ' ----- Build the parameterized query.
            sqlText = "SELECT FullName, AnnualFee FROM Customer WHERE ID = @CustID"
            commandWrapper = New SqlCommand(sqlText, linkToDB)
            commandWrapper.Parameters.AddWithValue("@CustID", ActiveCustomerID)
            commandWrapper.Connection = linkToDB

            ' ----- Process the query.
            Try
                customerReader = commandWrapper.ExecuteReader()
                customerReader.Read()
                CustomerName.Text = CStr(customerReader!FullName)
                CurrentFee.Text = Format(CDec(customerReader!AnnualFee), "0.00")
            Catch ex As Exception
                MessageBox.Show("Error occurred accessing customer record: " & ex.Message)
                ActOK.Enabled = False
                Return
            Finally
                If (customerReader IsNot Nothing) Then customerReader.Close()
            End Try
        End Using
        NewFee.Text = CurrentFee.Text
    End Sub

    Private Sub NewFee_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles NewFee.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Verify and save the data.
        Dim sqlText As String
        Dim commandWrapper As SqlCommand

        ' ----- The fee is required, and must be a valid number.
        If (IsNumeric(NewFee.Text) = False) Then
            MessageBox.Show("Please supply a valid annual fee.")
            Return
        End If

        ' ----- Don't bother if nothing changed.
        If (CDec(CurrentFee.Text) = CDec(NewFee.Text)) Then
            Me.DialogResult = DialogResult.Cancel
            Return
        End If

        ' ----- Save the new fee.
        sqlText = "UPDATE Customer SET AnnualFee = @NewFee WHERE ID = @CustID"
        commandWrapper = New SqlCommand(sqlText)
        commandWrapper.Parameters.AddWithValue("@NewFee", CDec(NewFee.Text))
        commandWrapper.Parameters.AddWithValue("@CustID", ActiveCustomerID)
        Try
            ExecuteSQL(commandWrapper)
        Catch ex As Exception
            MessageBox.Show("Error occurred updating annual fee: " & ex.Message)
            Return
        End Try

        ' ----- Success.
        Me.DialogResult = DialogResult.OK
    End Sub
End Class