﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 10 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class RenameCustomer
    Private ActiveCustomerID As Long

    Public Function PromptUser(ByVal whichCustomer As Long) As Boolean
        ' ----- Prompt the user to rename a customer record.
        ActiveCustomerID = whichCustomer
        If (Me.ShowDialog() = DialogResult.OK) Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub RenameCustomer_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim commandWrapper As SqlCommand

        ' ----- Retrieve the customer name.
        Using linkToDB As SqlConnection = OpenConnection()
            ' ----- Build the parameterized query.
            sqlText = "SELECT FullName FROM Customer WHERE ID = @CustID"
            commandWrapper = New SqlCommand(sqlText, linkToDB)
            commandWrapper.Parameters.AddWithValue("@CustID", ActiveCustomerID)

            ' ----- Process the query.
            Try
                CurrentName.Text = CStr(commandWrapper.ExecuteScalar)
            Catch ex As Exception
                MessageBox.Show("Error occurred accessing customer name: " & ex.Message)
                ActOK.Enabled = False
                Return
            End Try
        End Using
        NewName.Text = CurrentName.Text
    End Sub

    Private Sub NewName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles NewName.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Verify and save the data.
        Dim sqlText As String
        Dim commandWrapper As SqlCommand

        ' ----- The name is required.
        If (NewName.Text.Trim.Length = 0) Then
            MessageBox.Show("Please supply a new customer name.")
            Return
        End If

        ' ----- Don't bother if nothing changed.
        If (CurrentName.Text = NewName.Text) Then
            Me.DialogResult = DialogResult.Cancel
            Return
        End If

        ' ----- Make sure this name isn't used by another record.
        sqlText = "SELECT COUNT(*) FROM Customer WHERE ID <> @CustID " &
            " AND UPPER(FullName) = @TestName"
        commandWrapper = New SqlCommand(sqlText)
        commandWrapper.Parameters.AddWithValue("@CustID", ActiveCustomerID)
        commandWrapper.Parameters.AddWithValue("@TestName", NewName.Text.Trim.ToUpper)
        Try
            If (CInt(ExecuteSQLReturn(commandWrapper)) > 0) Then
                MessageBox.Show("That name is already in use.")
                Return
            End If
        Catch ex As Exception
            MessageBox.Show("Error occurred accessing customer name: " & ex.Message)
            Return
        End Try

        ' ----- Save the new name.
        sqlText = "UPDATE Customer SET FullName = @NewName WHERE ID = @CustID"
        commandWrapper = New SqlCommand(sqlText)
        commandWrapper.Parameters.AddWithValue("@NewName", NewName.Text.Trim)
        commandWrapper.Parameters.AddWithValue("@CustID", ActiveCustomerID)
        Try
            ExecuteSQL(commandWrapper)
        Catch ex As Exception
            MessageBox.Show("Error occurred updating customer name: " & ex.Message)
            Return
        End Try

        ' ----- Success.
        Me.DialogResult = DialogResult.OK
    End Sub
End Class