﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 12 - Visual Basic 
'       by Tim Patrick

Imports System.Data.SqlClient
Imports System.Transactions

Public Class AccountTransfer
    Private Const CheckingAccountID As String = "123456789"
    Private Const SavingsAccountID As String = "987654321"

    Private Sub AccountTransfer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Load in the account balances.
        RefreshBalances()
    End Sub

    Private Sub ActTransfer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActTransfer.Click
        ' ----- Initiate a transfer. First check for a valid transfer amount.
        If (IsNumeric(TransferAmount.Text) = False) Then
            MessageBox.Show("The transfer amount must be a valid number.")
            Return
        End If

        ' ----- Local or distributed?
        If (UseDistributed.Checked = True) Then
            If (TransferDistributed() = False) Then Return
        Else
            If (TransferLocal() = False) Then Return
        End If

        ' ----- Update the display.
        RefreshBalances()
    End Sub

    Private Sub TransferAmount_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles TransferAmount.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Public Function GetConnectionString() As String
        ' ----- Build a connection string for the active database.
        Dim builder As New SqlConnectionStringBuilder

        builder.DataSource = "(local)\SQLExpress"
        builder.InitialCatalog = "StepSample"
        builder.IntegratedSecurity = True
        Return builder.ConnectionString
    End Function

    Private Sub RefreshBalances()
        ' ----- Prepare the form.
        Dim result As Decimal
        Dim sqlText As String
        Dim accountCommand As SqlCommand

        Using linkToDB As SqlConnection = New SqlConnection(GetConnectionString())
            ' ----- Open the database.
            Try
                linkToDB.Open()
            Catch ex As Exception
                MessageBox.Show("Error accessing the database: " & ex.Message)
                Return
            End Try

            ' ----- Build a statement to get the balances.
            sqlText = "SELECT Balance FROM BankAccount WHERE AccountNumber = @ID"
            accountCommand = New SqlCommand(sqlText, linkToDB)

            ' ----- Get the checking account value.
            Try
                accountCommand.Parameters.AddWithValue("@ID", CheckingAccountID)
                result = CDec(accountCommand.ExecuteScalar())
                CheckingBalance.Text = Format(result, "Currency")
            Catch ex As Exception
                MessageBox.Show("Error retrieving checking account balance: " & ex.Message)
                Return
            End Try

            ' ----- Get the savings account value.
            Try
                accountCommand.Parameters("@ID").Value = SavingsAccountID
                result = CDec(accountCommand.ExecuteScalar())
                SavingsBalance.Text = Format(result, "Currency")
            Catch ex As Exception
                MessageBox.Show("Error retrieving savings account balance: " & ex.Message)
                Return
            End Try
        End Using
    End Sub

    Private Function TransferDistributed() As Boolean
        ' ----- Transfer money using a local transaction.
        Dim sqlText As String
        Dim toTransfer As Decimal
        Dim withdrawal As SqlCommand
        Dim deposit As SqlCommand

        ' ----- Retrieve the transfer amount.
        toTransfer = CDec(TransferAmount.Text)

        Using envelope As TransactionScope = New TransactionScope()
            ' ----- Create the withdrawal and deposit connections.
            Using sourceLink As SqlConnection = New SqlConnection(GetConnectionString())
                Using destLink As SqlConnection = New SqlConnection(GetConnectionString())
                    ' ----- Prepare and perform the withdrawal.
                    sqlText = "UPDATE BankAccount SET Balance = Balance - @ToTransfer " & _
                        "WHERE AccountNumber = @FromAccount"
                    withdrawal = New SqlCommand(sqlText, sourceLink)
                    withdrawal.Parameters.AddWithValue("@ToTransfer", toTransfer)

                    ' ----- Prepare and perform the deposit.
                    sqlText = "UPDATE BankAccount SET Balance = Balance + @ToTransfer " & _
                        "WHERE AccountNumber = @ToAccount"
                    deposit = New SqlCommand(sqlText, destLink)
                    deposit.Parameters.AddWithValue("@ToTransfer", toTransfer)

                    ' ----- Add in the target accounts.
                    If (OptFromChecking.Checked = True) Then
                        withdrawal.Parameters.AddWithValue("@FromAccount", CheckingAccountID)
                        deposit.Parameters.AddWithValue("@ToAccount", SavingsAccountID)
                    Else
                        withdrawal.Parameters.AddWithValue("@FromAccount", SavingsAccountID)
                        deposit.Parameters.AddWithValue("@ToAccount", CheckingAccountID)
                    End If

                    ' ----- Perform the transfer.
                    Try
                        ' ----- Perform the withdrawal.
                        sourceLink.Open()
                        withdrawal.ExecuteNonQuery()

                        ' ----- Perform the deposit.
                        destLink.Open()
                        deposit.ExecuteNonQuery()

                        ' ----- Transfer complete. Commit the transaction.
                        envelope.Complete()
                    Catch ex As Exception
                        MessageBox.Show("Error transferring funds: " & ex.Message)
                        Return False
                    End Try
                End Using
            End Using
        End Using

        ' ----- Successful transaction.
        Return True
    End Function

    Private Function TransferLocal() As Boolean
        ' ----- Transfer money using a local transaction.
        Dim sqlText As String
        Dim toTransfer As Decimal
        Dim withdrawal As SqlCommand
        Dim deposit As SqlCommand
        Dim envelope As SqlTransaction

        ' ----- Retrieve the transfer amount.
        toTransfer = CDec(TransferAmount.Text)

        Using linkToDB As SqlConnection = New SqlConnection(GetConnectionString())
            Try
                ' ----- The database must be opened to create the transaction.
                linkToDB.Open()

                ' ----- Prepare a transaction to surround the transfer.
                envelope = linkToDB.BeginTransaction()
            Catch ex As Exception
                MessageBox.Show("Error accessing the database: " & ex.Message)
                Return False
            End Try

            ' ----- Prepare and perform the withdrawal.
            sqlText = "UPDATE BankAccount SET Balance = Balance - @ToTransfer " & _
                "WHERE AccountNumber = @FromAccount"
            withdrawal = New SqlCommand(sqlText, linkToDB, envelope)
            withdrawal.Parameters.AddWithValue("@ToTransfer", toTransfer)
            If (OptFromChecking.Checked = True) Then
                withdrawal.Parameters.AddWithValue("@FromAccount", CheckingAccountID)
            Else
                withdrawal.Parameters.AddWithValue("@FromAccount", SavingsAccountID)
            End If

            ' ----- Prepare and perform the deposit.
            sqlText = "UPDATE BankAccount SET Balance = Balance + @ToTransfer " & _
                "WHERE AccountNumber = @ToAccount"
            deposit = New SqlCommand(sqlText, linkToDB, envelope)
            deposit.Parameters.AddWithValue("@ToTransfer", toTransfer)
            If (OptFromChecking.Checked = True) Then
                deposit.Parameters.AddWithValue("@ToAccount", SavingsAccountID)
            Else
                deposit.Parameters.AddWithValue("@ToAccount", CheckingAccountID)
            End If

            ' ----- Perform the transfer.
            Try
                withdrawal.ExecuteNonQuery()
                deposit.ExecuteNonQuery()
                envelope.Commit()
            Catch ex As Exception
                MessageBox.Show("Error transferring funds: " & ex.Message)

                ' ----- Do a rollback instead.
                Try
                    envelope.Rollback()
                Catch ex2 As Exception
                    MessageBox.Show("Error cancelling the transaction: " & ex2.Message)
                End Try
                Return False
            End Try
        End Using

        ' ----- Successful transaction.
        Return True
    End Function
End Class
