﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 15 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.Common
Imports System.Data.EntityClient
Imports System.Data.SqlClient
Imports System.Data.Objects
Imports System.Data.Metadata.Edm

Public Class EntityQuery
    Private ActiveContext As SalesOrderEntities

    Private Sub ActSimpleRetrieval_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSimpleRetrieval.Click
        ' ----- Simple retrieval of all entities in a set.
        Dim query As ObjectSet(Of Customer)

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        Try
            ' ----- Retrieve the customer entities directly.
            ActiveContext = New SalesOrderEntities(GetConnectionString())
            query = ActiveContext.CreateObjectSet(Of Customer)()

            ' ----- Display the results to the user.
            DisplayResults.DataSource = query
            MessageBox.Show(query.Count & " result(s)")
        Catch ex As Exception
            MessageBox.Show("Error during record retrieval: " & ex.Message)
        End Try
    End Sub

    Private Sub ActSingleEntity_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSingleEntity.Click
        ' ----- Query of single entity.
        Dim sqlText As String
        Dim query As ObjectQuery(Of Customer)

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        Try
            ' ----- Retrieve the customer entities via a query.
            ActiveContext = New SalesOrderEntities(GetConnectionString())
            sqlText = "SELECT VALUE Customer FROM Customers " &
                "AS Customer ORDER BY Customer.FullName DESC"
            query = New ObjectQuery(Of Customer)(sqlText, ActiveContext)

            ' ----- Display the results to the user.
            DisplayResults.DataSource = query
            MessageBox.Show(query.Count & " result(s)")
        Catch ex As Exception
            MessageBox.Show("Error during single record retrieval: " & ex.Message)
        End Try
    End Sub

    Private Sub ActAnonymousType_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAnonymousType.Click
        ' ----- Query that returns a custom (anonymous) type.
        Dim sqlText As String
        Dim query As ObjectQuery(Of DbDataRecord)

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        Try
            ' ----- Retrieve the entity data via a query.
            ActiveContext = New SalesOrderEntities(GetConnectionString())
            sqlText = "SELECT CU.FullName, OE.ID, OE.OrderDate, OE.Total " &
                "FROM Customers AS CU INNER JOIN " &
                "OrderEntries AS OE ON CU.ID = OE.Customer"
            query = New ObjectQuery(Of DbDataRecord)(sqlText, ActiveContext)

            ' ----- Display the results to the user.
            DisplayResults.DataSource = query
            MessageBox.Show(query.Count & " result(s)")
        Catch ex As Exception
            MessageBox.Show("Error during anonymous record retrieval: " & ex.Message)
        End Try
    End Sub

    Private Sub ActNavigationProperty_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActNavigationProperty.Click
        ' ----- Query that includes use of a navagation property.
        Dim sqlText As String
        Dim query As ObjectQuery(Of DbDataRecord)

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        Try
            ' ----- Retrieve the data by following the navigation property.
            ActiveContext = New SalesOrderEntities(GetConnectionString())
            sqlText = "SELECT CU.FullName, CU.State.Abbreviation " &
                "FROM Customers AS CU"
            query = New ObjectQuery(Of DbDataRecord)(sqlText, ActiveContext)

            ' ----- Display the results to the user.
            DisplayResults.DataSource = query
            MessageBox.Show(query.Count & " result(s)")
        Catch ex As Exception
            MessageBox.Show("Error during navigation property retrieval: " & ex.Message)
        End Try
    End Sub

    Private Sub ActDataTable_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDataTable.Click
        ' ----- Query that returns an ADO.NET DataTable.
        Dim oneRow As DataRow
        Dim resultsAsTable As DataTable
        Dim query As EntityCommand
        Dim results As EntityDataReader = Nothing
        Dim sqlText As String

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        ' ----- Build a storage area for the results.
        resultsAsTable = New DataTable
        resultsAsTable.Columns.Add("CustomerID", GetType(Long))
        resultsAsTable.Columns.Add("CustomerName", GetType(String))
        resultsAsTable.Columns.Add("AnnualFee", GetType(Decimal))

        ' ----- Connect to the entity provider.
        Using linkToDB As New EntityConnection(GetConnectionString())
            Try
                linkToDB.Open()
            Catch ex As Exception
                MessageBox.Show("Error opening the data connection: " & ex.Message)
                Return
            End Try

            ' ----- Retrieve the data via a parameterized query.
            sqlText = "SELECT CU.ID, CU.FullName, CU.AnnualFee " &
                "FROM SalesOrderEntities.Customers AS CU " &
                "WHERE CU.AnnualFee >= @MinFee ORDER BY CU.FullName"
            Try
                query = New EntityCommand(sqlText, linkToDB)
                query.Parameters.AddWithValue("MinFee", 200)
                results = query.ExecuteReader(CommandBehavior.SequentialAccess)
            Catch ex As Exception
                MessageBox.Show("Error processing parameterized query: " & ex.Message)
                Return
            End Try

            Try
                ' ----- Move each row into the DataTable.
                Do While (results.Read() = True)
                    oneRow = resultsAsTable.NewRow()
                    oneRow!CustomerID = CLng(results!ID)
                    oneRow!CustomerName = CStr(results!FullName)
                    oneRow!AnnualFee = CDec(results!AnnualFee)
                    resultsAsTable.Rows.Add(oneRow)
                Loop
            Catch ex As Exception
                MessageBox.Show("Error saving query results: " & ex.Message)
                Return
            Finally
                results.Close()
            End Try

            ' ----- Display the results to the user.
            Try
                DisplayResults.DataSource = resultsAsTable
                MessageBox.Show(resultsAsTable.Rows.Count & " result(s)")
            Catch ex As Exception
                MessageBox.Show("Error displaying query results: " & ex.Message)
            End Try
        End Using
    End Sub

    Public Function GetConnectionString() As String
        ' ----- Build a connection string for the active database.
        Dim sqlPortion As New SqlConnectionStringBuilder
        Dim builder As New EntityConnectionStringBuilder

        ' ----- Build the provider portion of the connection string.
        sqlPortion.DataSource = "(local)\SQLExpress"
        sqlPortion.InitialCatalog = "StepSample"
        sqlPortion.IntegratedSecurity = True

        ' ----- Build the Entity Client connection string.
        builder.Metadata = "res://*/SalesOrder.csdl|res://*/SalesOrder.ssdl|res://*/SalesOrder.msl"
        builder.Provider = "System.Data.SqlClient"
        builder.ProviderConnectionString = sqlPortion.ConnectionString
        Return builder.ConnectionString
    End Function

    Private Sub ActProcess_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActProcess.Click
        ' ----- Let the user supply a custom query.
        Dim sqlText As String
        Dim query As ObjectQuery(Of DbDataRecord)

        ' ----- Check for a valid query.
        If (CustomQuery.Text.Trim.Length = 0) Then
            MessageBox.Show("Please supply a query.")
            Return
        End If

        ' ----- Clear any previous results.
        DisplayResults.DataSource = Nothing

        Try
            ' ----- Retrieve the custom query content.
            ActiveContext = New SalesOrderEntities(GetConnectionString())
            sqlText = CustomQuery.Text
            query = New ObjectQuery(Of DbDataRecord)(sqlText, ActiveContext)

            ' ----- Display the results to the user.
            DisplayResults.DataSource = query
            MessageBox.Show(query.Count & " result(s)")
        Catch ex As Exception
            MessageBox.Show("Error processing query: " & ex.Message)
        End Try
    End Sub
End Class
