﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 16 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.Objects
Imports System.Data.SqlClient
Imports System.Data.EntityClient

Public Class CustomerEditor
    Private ActiveContext As SalesOrderEntities

    Private Sub AllCustomers_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AllCustomers.SelectedIndexChanged
        ' ----- Enable the buttons as needed.
        ActEdit.Enabled = CBool(AllCustomers.SelectedIndex <> -1)
        ActDelete.Enabled = CBool(AllCustomers.SelectedIndex <> -1)
    End Sub

    Private Sub CustomerEditor_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' ----- Close the context.
        If (Me.ActiveContext IsNot Nothing) Then Me.ActiveContext.Dispose()
    End Sub

    Private Sub CustomerEditor_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim scanCustomer As Customer

        ' ----- Create the context. It will remain during the entire program.
        Try
            Me.ActiveContext = New SalesOrderEntities(GetConnectionString())
        Catch ex As Exception
            MessageBox.Show("Error connecting to database: " & ex.Message)
            End
        End Try

        ' ----- Display the existing customers.
        Try
            For Each scanCustomer In Me.ActiveContext.Customers
                AllCustomers.Items.Add(New ItemData(scanCustomer.FullName, scanCustomer.ID))
            Next scanCustomer
        Catch ex As Exception
            MessageBox.Show("Error loading customers: " & ex.Message)
        End Try
    End Sub

    Private Sub ActAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAdd.Click
        ' ----- Add a new customer to the database.
        Dim newCustomer As Customer = Nothing

        ' ----- Prompt the user.
        If ((New CustomerDetail).EditCustomer(newCustomer, Me.ActiveContext) = False) Then Return

        ' ----- Add the customer to the list.
        AllCustomers.Items.Add(New ItemData(newCustomer.FullName, newCustomer.ID))
    End Sub

    Private Sub ActEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActEdit.Click
        ' ----- Edit an existing customer.
        Dim whichCustomer As Customer

        ' ----- Don't bother if no customer is selected.
        If (AllCustomers.SelectedIndex = -1) Then Return

        ' ----- Locate the customer record.
        whichCustomer = Me.ActiveContext.Customers.Where("it.ID = @lookupID",
            New ObjectParameter("lookupID", ItemData.GetItemData(
            AllCustomers.SelectedItem))).First()

        ' ----- Prompt the user.
        If ((New CustomerDetail).EditCustomer(whichCustomer, Me.ActiveContext) = False) Then Return

        ' ----- Update the customer in the list.
        AllCustomers.Items(AllCustomers.SelectedIndex) =
            New ItemData(whichCustomer.FullName, whichCustomer.ID)
    End Sub

    Private Sub ActDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDelete.Click
        ' ----- Delete a customer record.
        Dim whichCustomer As Customer = Nothing

        ' ----- Don't bother if no customer is selected.
        If (AllCustomers.SelectedIndex = -1) Then Return

        ' ----- Locate the customer record.
        whichCustomer = Me.ActiveContext.Customers.Where("it.ID = @lookupID",
            New ObjectParameter("lookupID", ItemData.GetItemData(
            AllCustomers.SelectedItem))).First()
        If (whichCustomer Is Nothing) Then Return

        ' ----- Confirm with the user.
        If (MessageBox.Show("Really delete the customer '" &
            whichCustomer.FullName & "'?", "Delete Customer",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return

        ' ----- If this customer has orders, don't delete it.
        If (whichCustomer.OrderEntries.Count > 0) Then
            MessageBox.Show("You cannot delete this customer because it has orders.")
            Return
        End If

        ' ----- Delete the customer from the database.
        Try
            Me.ActiveContext.DeleteObject(whichCustomer)
            Me.ActiveContext.SaveChanges()
        Catch ex As Exception
            MessageBox.Show("Error deleting customer record: " & ex.Message)
            Return
        End Try

        ' ----- Delete the customer from the display.
        AllCustomers.Items.RemoveAt(AllCustomers.SelectedIndex)
        ActEdit.Enabled = False
        ActDelete.Enabled = False
    End Sub

    Private Function GetConnectionString() As String
        ' ----- Build a connection string for the active database.
        Dim sqlPortion As New SqlConnectionStringBuilder
        Dim builder As New EntityConnectionStringBuilder

        ' ----- Build the provider portion of the connection string.
        sqlPortion.DataSource = "(local)\SQLExpress"
        sqlPortion.InitialCatalog = "StepSample"
        sqlPortion.IntegratedSecurity = True

        ' ----- Build the Entity Client connection string.
        builder.Metadata = "res://*/SalesOrder.csdl|res://*/SalesOrder.ssdl|res://*/SalesOrder.msl"
        builder.Provider = "System.Data.SqlClient"
        builder.ProviderConnectionString = sqlPortion.ConnectionString
        Return builder.ConnectionString
    End Function
End Class
