﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 18 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class OrderViewer
    ' ----- Fields used to monitor the active database content.
    Private CustomerSet As New DataSet

    Private Sub OrderViewer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim retrieveCommand As SqlCommand
        Dim linkToDB As SqlConnection
        Dim orderAdapter As New SqlDataAdapter

        ' ----- Open the database.
        linkToDB = New SqlConnection(GetConnectionString())

        ' ----- Build the command for the customer and order records.
        retrieveCommand = New SqlCommand("SELECT * FROM Customer;" & _
            "SELECT * FROM OrderEntry", linkToDB)
        orderAdapter.SelectCommand = retrieveCommand

        ' ----- Using the basic external-internal syntax is quick.
        orderAdapter.TableMappings.Add("Table", "Customer")
        orderAdapter.TableMappings.Add("Table1", "Order")

        ' ----- Load the data from the database into local storage.
        Try
            CustomerSet = New DataSet
            orderAdapter.Fill(CustomerSet)
        Catch ex As Exception
            MessageBox.Show("Error occurred loading customer orders: " & ex.Message)
        End Try
    End Sub

    Private Sub ActView_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActView.Click
        ' ----- Try to display one or more customer orders.
        Dim customerTable As DataTable
        Dim orderTable As DataTable

        ' ----- Check for a valid customer ID if in ID mode.
        If (OptOneCustomer.Checked = True) Then
            If (IsNumeric(CustomerID.Text) = False) Then
                MessageBox.Show("Please supply a valid customer ID.")
                Return
            End If
        End If

        ' ----- Clear any previous results.
        AllOrders.DataSource = Nothing

        ' ----- Locate the customer and order tables.
        customerTable = CustomerSet.Tables!Customer
        orderTable = CustomerSet.Tables!Order

        ' ----- Build an ad hoc table of status codes.
        Dim statusTable = {New With {.Code = "P", .Description = "Active Order"},
            New With {.Code = "C", .Description = "Completed / Shipped"},
            New With {.Code = "X", .Description = "Canceled"}}

        ' ----- Retrieve all customer orders.
        Dim result = From cu In customerTable.AsEnumerable(),
                     ord In orderTable.AsEnumerable(),
                     sts In statusTable
                     Where cu.Field(Of Long)("ID") = ord.Field(Of Long)("Customer") _
                     And ord.Field(Of String)("StatusCode") = sts.Code
                     Select CustomerID = cu.Field(Of Long)("ID"),
                     CustomerName = cu.Field(Of String)("FullName"),
                     OrderID = ord.Field(Of Long)("ID"),
                     OrderStatus = sts.Description,
                     OrderDate = ord.Field(Of Date)("OrderDate"),
                     OrderTotal = ord.Field(Of Decimal)("Total")
                     Order By CustomerName, OrderID

        ' ----- If this is a single customer query, apply the filter.
        If (OptOneCustomer.Checked = True) Then
            ' ----- Filter and display the orders. Try the extension
            '       method way of filtering.
            Dim result2 = result.Where(Function(ord) ord.CustomerID = CLng(CustomerID.Text))
            AllOrders.DataSource = result2.ToList()
        Else
            ' ----- Just display the original full results.
            AllOrders.DataSource = result.ToList()
        End If
    End Sub

    Public Function GetConnectionString() As String
        ' ----- Build a connection string for the active database.
        Dim builder As New SqlConnectionStringBuilder

        builder.DataSource = "(local)\SQLExpress"
        builder.InitialCatalog = "StepSample"
        builder.IntegratedSecurity = True
        Return builder.ConnectionString
    End Function

    Private Sub CustomerID_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles CustomerID.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub CustomerOptions_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
            OptAllCustomers.CheckedChanged, OptOneCustomer.CheckedChanged
        ' ----- Enable the customer ID field if needed.
        If (CType(sender, RadioButton).Checked = True) Then
            CustomerID.Enabled = OptOneCustomer.Checked
        End If
    End Sub
End Class
