﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 19 - Visual Basic Version
'       by Tim Patrick

Public Class OrderViewer
    ' ----- Fields used to monitor the active database content.
    Private OrderContext As SalesOrderEntities

    Private Sub OrderViewer_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' ----- Clean up.
        If (OrderContext IsNot Nothing) Then OrderContext.Dispose()
    End Sub

    Private Sub OrderViewer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Open the context to the set of customer orders.
        OrderContext = New SalesOrderEntities(GetConnectionString())
    End Sub

    Private Sub ActView_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActView.Click
        ' ----- Try to display one or more customer orders.

        ' ----- Check for a valid customer ID if in ID mode.
        If (OptOneCustomer.Checked = True) Then
            If (IsNumeric(CustomerID.Text) = False) Then
                MessageBox.Show("Please supply a valid customer ID.")
                Return
            End If
        End If

        ' ----- Clear any previous results.
        AllOrders.DataSource = Nothing

        ' ----- Build an ad hoc table of status codes.
        Dim statusTable = {New With {.Code = "P", .Description = "Active Order"},
            New With {.Code = "C", .Description = "Completed / Shipped"},
            New With {.Code = "X", .Description = "Canceled"}}

        ' ----- Retrieve all customer orders.
        Dim result = From cu In OrderContext.Customers,
                     ord In OrderContext.OrderEntries
                     Where cu.ID = ord.Customer
                     Select CustomerID = cu.ID,
                     CustomerName = cu.FullName,
                     OrderID = ord.ID,
                     OrderDate = ord.OrderDate,
                     OrderTotal = ord.Total,
                     ord.StatusCode
                     Order By CustomerName, OrderID

        ' ----- Add in the status code.
        Dim result2 = From cu In result.ToArray(), sts In statusTable
                      Where cu.StatusCode = sts.Code
                      Select cu.CustomerID, cu.CustomerName, cu.OrderID,
                      OrderStatus = sts.Description, cu.OrderDate, cu.OrderTotal

        ' ----- If this is a single customer query, apply the filter.
        If (OptOneCustomer.Checked = True) Then
            ' ----- Filter and display the orders. Try the extension
            '       method way of filtering.
            Dim result3 = result2.Where(Function(ord) ord.CustomerID = CLng(CustomerID.Text))
            AllOrders.DataSource = result3.ToList()
        Else
            ' ----- Just display the original full results.
            AllOrders.DataSource = result2.ToList()
        End If
    End Sub

    Private Sub CustomerID_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles CustomerID.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub CustomerOptions_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
            OptAllCustomers.CheckedChanged, OptOneCustomer.CheckedChanged
        ' ----- Enable the customer ID field if needed.
        If (CType(sender, RadioButton).Checked = True) Then
            CustomerID.Enabled = OptOneCustomer.Checked
        End If
    End Sub
End Class
