﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 3 - Visual Basic Version
'       by Tim Patrick

Public Class AccountManager
    Private WithEvents CustomerAccounts As DataTable

    Private Sub AccountManager_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim oneColumn As DataColumn
        Dim scanRow As DataRow

        ' ----- Build the table of account details.
        CustomerAccounts = New DataTable("Account")

        ' ----- ID is the primary key.
        oneColumn = CustomerAccounts.Columns.Add("ID", GetType(Long))
        oneColumn.AutoIncrement = True
        oneColumn.ReadOnly = True
        CustomerAccounts.PrimaryKey = {oneColumn}

        ' ----- Enforce unique account names, up to 30 characters.
        oneColumn = CustomerAccounts.Columns.Add("FullName", GetType(String))
        oneColumn.MaxLength = 30
        oneColumn.AllowDBNull = False
        oneColumn.Unique = True

        ' ----- Accounts are active by default.
        oneColumn = CustomerAccounts.Columns.Add("Active", GetType(Boolean))
        oneColumn.DefaultValue = True

        ' ----- The last two columns are optional.
        CustomerAccounts.Columns.Add("AnnualFee", GetType(Decimal))
        CustomerAccounts.Columns.Add("StartDate", GetType(Date))

        ' ----- Build some sample data rows.
        CustomerAccounts.Rows.Add({1&, "Blue Yonder Airlines", True, 500@, #1/1/2007#})
        CustomerAccounts.Rows.Add({2&, "Fourth Coffee", True, 350@, #7/25/2009#})
        CustomerAccounts.Rows.Add({3&, "Wingtip Toys", False})
        CustomerAccounts.AcceptChanges()

        ' ----- Add each row to the display list as well.
        For Each scanRow In CustomerAccounts.Rows
            AllAccounts.Items.Add(scanRow)
        Next scanRow
    End Sub

    Private Sub ActAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAdd.Click
        ' ----- Add a new account.
        Dim editRecord As DataRow
        Dim addComplete As Boolean = False

        ' ----- Keep prompting the user until the data is valid.
        editRecord = CustomerAccounts.NewRow()
        Do While (addComplete = False)
            ' ----- Prompt the user.
            editRecord.ClearErrors()
            If ((New AccountDetail).EditAccount(editRecord,
                CustomerAccounts) = False) Then Return

            ' ----- The user completed edits to the row. Commit the changes.
            Try
                CustomerAccounts.Rows.Add(editRecord)
                CustomerAccounts.AcceptChanges()
                addComplete = True
            Catch ex As Exception
                MessageBox.Show("The account could not be added due to the following error: " &
                    ex.Message)
            End Try
        Loop

        ' ----- Add this item to the display list.
        AllAccounts.Items.Add(editRecord)
    End Sub

    Private Sub ActEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActEdit.Click
        ' ----- Edit the selected account.
        Dim editRecord As DataRow
        Dim editComplete As Boolean = False

        ' ----- Ignore if no item is active.
        If (AllAccounts.SelectedIndex = -1) Then Return
        editRecord = CType(AllAccounts.SelectedItem, DataRow)

        ' ----- Keep prompting the user until the data is valid.
        Do While (editComplete = False)
            ' ----- Prompt the user.
            editRecord.ClearErrors()
            If ((New AccountDetail).EditAccount(editRecord, CustomerAccounts) = False) Then
                CustomerAccounts.RejectChanges()
                AllAccounts.Refresh()
                Return
            End If

            ' ----- The user completed edits to the row. Commit the changes.
            Try
                CustomerAccounts.AcceptChanges()
                editComplete = True
            Catch ex As Exception
                MessageBox.Show("The account could not be changed due to the following error: " &
                    ex.Message)
            End Try
        Loop

        ' ----- Refresh the item in the display list.
        AllAccounts.Refresh()
    End Sub

    Private Sub ActDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDelete.Click
        ' ----- Delete the selected account.
        Dim editRecord As DataRow

        ' ----- Ignore if no item is active.
        If (AllAccounts.SelectedIndex = -1) Then Return
        editRecord = CType(AllAccounts.SelectedItem, DataRow)

        ' ----- Prompt the user.
        If (MessageBox.Show("Really delete the selected account?", Me.Text,
            MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) <>
            DialogResult.Yes) Then Return

        ' ----- Remove the record from the table.
        Try
            editRecord.Delete()
            CustomerAccounts.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("The account could not be deleted due to the following error: " &
                ex.Message)
            CustomerAccounts.RejectChanges()
            Return
        End Try

        ' ----- Remove from the display list.
        AllAccounts.Items.Remove(editRecord)
        ActEdit.Enabled = False
        ActDelete.Enabled = False
    End Sub

    Private Sub ActClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActClose.Click
        ' ----- End the sample.
        Me.Close()
    End Sub

    Private Sub AllAccounts_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles AllAccounts.DoubleClick
        ' ----- Same as the edit button.
        If (ActEdit.Enabled = True) Then ActEdit.PerformClick()
    End Sub

    Private Sub AllAccounts_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles AllAccounts.DrawItem
        ' ----- Draw a single display item.
        Dim useBrush As System.Drawing.Brush
        Dim itemDetail As DataRow
        Dim ellipsesFormat As StringFormat
        Dim activeColFont As Font
        Const ActiveCharacter As Integer = &HFC   ' Wingdings checkmark
        Const InactiveCharacter As Integer = &HFB ' Wingdings x-mark

        ' ----- Ignore undrawable items.
        If (e.Index = -1) Then Return

        ' ----- Draw the background of the item.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) _
            Then useBrush = SystemBrushes.HighlightText _
            Else useBrush = SystemBrushes.WindowText
        e.DrawBackground()

        ' ----- Extract the detail from the list.
        itemDetail = CType(AllAccounts.Items(e.Index), DataRow)

        ' ----- Some data may not fit.
        ellipsesFormat = New StringFormat
        ellipsesFormat.Trimming = StringTrimming.EllipsisCharacter

        ' ----- Draw the account name.
        e.Graphics.DrawString(CStr(itemDetail!FullName), e.Font, useBrush,
            New Rectangle(ColFullName.Left - ColActive.Left,
            e.Bounds.Top, ColAnnualFee.Left - ColFullName.Left - 5,
            e.Bounds.Height), ellipsesFormat)

        ' ----- Draw the annual fee, if needed.
        If (IsDBNull(itemDetail!AnnualFee) = False) Then _
            e.Graphics.DrawString(Format(CDec(itemDetail!AnnualFee), "Currency"),
            e.Font, useBrush, ColAnnualFee.Left - ColActive.Left, e.Bounds.Top)

        ' ----- Draw the start date, if needed.
        If (IsDBNull(itemDetail!StartDate) = False) Then _
            e.Graphics.DrawString(Format(CDate(itemDetail!StartDate), "Short Date"),
            e.Font, useBrush, ColStartDate.Left - ColActive.Left, e.Bounds.Top) 

        ' ----- Use a special font to show the active status.
        activeColFont = New Font("Wingdings", AllAccounts.Font.Size)

        ' ----- The display color for the active status differs as well.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) Then
            useBrush = SystemBrushes.HighlightText
        ElseIf (CBool(itemDetail!Active) = True) Then
            useBrush = Brushes.Green
        Else
            useBrush = Brushes.Red
        End If

        ' ----- Indicate the active status of this row.
        If (CBool(itemDetail!Active) = True) Then
            ' ----- Active account.
            e.Graphics.DrawString(ChrW(ActiveCharacter), activeColFont,
                useBrush, 0, e.Bounds.Top)
        Else
            ' ----- Inactive account.
            e.Graphics.DrawString(ChrW(InactiveCharacter), activeColFont,
                useBrush, 0, e.Bounds.Top)
        End If
        activeColFont.Dispose()

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
    End Sub

    Private Sub AllAccounts_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AllAccounts.SelectedIndexChanged
        ' ----- Enable action buttons as needed.
        ActEdit.Enabled = (AllAccounts.SelectedIndex <> -1)
        ActDelete.Enabled = (AllAccounts.SelectedIndex <> -1)
    End Sub

    Private Sub CustomerAccounts_ColumnChanging(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles CustomerAccounts.ColumnChanging
        ' ----- Some columns have special business rules.
        If (e.Column.ColumnName = "AnnualFee") Then
            ' ----- Annual fee may not be negative.
            If (IsDBNull(e.ProposedValue) = False) Then
                If (CDec(e.ProposedValue) < 0@) Then _
                    e.Row.SetColumnError(e.Column, "Annual fee may not be negative.")
            End If
        ElseIf (e.Column.ColumnName = "StartDate") Then
            ' ----- Start date must be on or before today.
            If (IsDBNull(e.ProposedValue) = False) Then
                If (CDate(e.ProposedValue).Date > Today) Then _
                    e.Row.SetColumnError(e.Column, "Start date must occur on or before today.")
            End If
        End If
    End Sub

    Private Sub CustomerAccounts_RowChanging(ByVal sender As Object, ByVal e As System.Data.DataRowChangeEventArgs) Handles CustomerAccounts.RowChanging
        ' ----- The start date is required for active accounts.
        If (e.Row.HasVersion(DataRowVersion.Proposed) = True) Then
            If (CBool(e.Row("Active", DataRowVersion.Proposed)) = True) And
                    (IsDBNull(e.Row("StartDate", DataRowVersion.Proposed)) = True) Then
                e.Row.RowError = "Active accounts must have a valid start date."
            End If
        End If
    End Sub
End Class
