﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 6 - Visual Basic Version
'       by Tim Patrick

Public Class Aggregates
    Private ExampleData As DataSet

    Private Sub Aggregates_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Fill in the list of compute functions.
        ComputeFunction.Items.Clear()
        ComputeFunction.Items.AddRange(
            Split("Sum,Avg,Min,Max,Count,StDev,Var", ","))
        ComputeFunction.SelectedIndex = 0

        ' ----- Fill in the data type choices.
        ParentDataType.Items.Clear()
        ParentDataType.Items.AddRange(
            Split("Boolean,Byte,Char,DateTime,Decimal,Double,Int16,Int32," &
            "Int64,SByte,Single,String,TimeSpan,UInt16,UInt32,UInt64", ","))
        ParentDataType.SelectedIndex = 0

        ChildDataType.Items.Clear()
        ChildDataType.Items.AddRange(
            Split("Boolean,Byte,Char,DateTime,Decimal,Double,Int16,Int32," &
            "Int64,SByte,Single,String,TimeSpan,UInt16,UInt32,UInt64", ","))
        ChildDataType.SelectedIndex = 0

        ' ----- Fill in the tables with default data.
        ResetDataTables()

        ' ----- Update the compute feature's view of columns.
        RefreshComputeColumns()
    End Sub

    Private Sub ActParentColumn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActParentColumn.Click
        ' ----- Check for valid data.
        If (ParentName.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a column name.")
            ParentName.Focus()
            Return
        End If
        If (ParentExpression.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide an expression.")
            ParentExpression.Focus()
            Return
        End If

        ' ----- Add the new column.
        Try
            ExampleData.Tables("Parent").Columns.Add(ParentName.Text.Trim,
                Type.GetType("System." & ParentDataType.SelectedItem.ToString()),
                ParentExpression.Text.Trim)
        Catch ex As Exception
            MessageBox.Show("Could not add the column: " & ex.Message)
            Return
        End Try

        ' ----- Refresh the columns used for quick computing.
        RefreshComputeColumns()

        ' ----- Reset for another column.
        ParentName.Clear()
        ParentDataType.SelectedIndex = 0
        ParentExpression.Clear()
    End Sub

    Private Sub ActChildColumn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActChildColumn.Click
        ' ----- Check for valid data.
        If (ChildName.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a column name.")
            ChildName.Focus()
            Return
        End If
        If (ChildExpression.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide an expression.")
            ChildExpression.Focus()
            Return
        End If

        ' ----- Add the new column.
        Try
            ExampleData.Tables("Child").Columns.Add(ChildName.Text.Trim,
                Type.GetType("System." & ChildDataType.SelectedItem.ToString()),
                ChildExpression.Text.Trim)
        Catch ex As Exception
            MessageBox.Show("Could not add the column: " & ex.Message)
            Return
        End Try

        ' ----- Refresh the columns used for quick computing.
        RefreshComputeColumns()

        ' ----- Reset for another column.
        ChildName.Clear()
        ChildDataType.SelectedIndex = 0
        ChildExpression.Clear()
    End Sub

    Private Sub ActCompute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActCompute.Click
        ' ----- Run a quick computation against a table column.
        Dim isParent As Boolean
        Dim columnName As String
        Dim delimIndex As Integer
        Dim expression As String
        Dim result As Object
        Dim whichTable As DataTable

        ' ----- Get the selection components.
        columnName = ComputeColumn.SelectedItem.ToString()
        delimIndex = columnName.IndexOf(".")
        isParent = (columnName.Substring(0, delimIndex) = "Parent")
        columnName = columnName.Substring(delimIndex + 1)

        ' ----- Build the expression.
        expression = ComputeFunction.SelectedItem.ToString() & "(" &
            columnName & ")"

        ' ----- Get the table that will process the expression.
        If (isParent = True) Then
            whichTable = ExampleData.Tables("Parent")
        Else
            whichTable = ExampleData.Tables("Child")
        End If

        ' ----- Process the expression.
        Try
            result = whichTable.Compute(expression, "")
        Catch ex As Exception
            MessageBox.Show("Could not compute the column: " & ex.Message)
            Return
        End Try

        ' ----- Display the results.
        If (IsDBNull(result) = True) Then
            MessageBox.Show("NULL")
        Else
            MessageBox.Show(result.ToString())
        End If
    End Sub

    Private Sub ActReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActReset.Click
        ' ----- Start over with fresh data.
        ResetDataTables()
    End Sub

    Private Sub TextFields_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles _
            ParentName.Enter, ChildName.Enter, ParentExpression.Enter, ChildExpression.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Function GetParentTable() As DataTable
        ' ----- Build a sample parent table.
        Dim sourceLines() As String
        Dim oneLine As String
        Dim lineFields() As String
        Dim oneRow As DataRow
        Dim result As New DataTable("Parent")

        ' ----- Add the column definitions.
        result.Columns.Add("ID", GetType(String))
        result.Columns.Add("StateName", GetType(String))
        result.Columns.Add("AreaSqMiles", GetType(Integer))
        result.Columns.Add("Statehood", GetType(Date))
        result.Columns.Add("Capital", GetType(String))

        ' ----- Add some sample data. The "States" resource has the
        '       following comma-delimited fields:
        '         ID (a.k.a., State Abbreviation)
        '         State Name
        '         Area in Square Miles
        '         Date of Statehood (optional)
        '         Capital (optional)
        sourceLines = Split(My.Resources.States, vbCrLf)
        For Each oneLine In sourceLines
            ' ----- Add a table row.
            lineFields = Split(oneLine, ",")
            oneRow = result.NewRow()
            oneRow!ID = lineFields(0)
            oneRow!StateName = lineFields(1)
            oneRow!AreaSqMiles = CInt(lineFields(2))
            If (lineFields(3).Length > 0) Then oneRow!Statehood = CDate(lineFields(3))
            If (lineFields(4).Length > 0) Then oneRow!Capital = lineFields(4)
            result.Rows.Add(oneRow)
        Next oneLine

        ' ----- Finished.
        Return result
    End Function

    Private Function GetChildTable() As DataTable
        ' ----- Build a sample child table.
        Dim sourceLines() As String
        Dim oneLine As String
        Dim lineFields() As String
        Dim oneRow As DataRow
        Dim result As New DataTable("Child")

        ' ----- Add the column definitions.
        result.Columns.Add("ParentID", GetType(String))
        result.Columns.Add("CountyName", GetType(String))
        result.Columns.Add("Population2009", GetType(Integer))
        result.Columns.Add("Births2009", GetType(Integer))
        result.Columns.Add("Deaths2009", GetType(Integer))

        ' ----- Add some sample data. The "Counties" resource has the
        '       following comma-delimited fields:
        '         Parent ID (a.k.a., State Abbreviation)
        '         County Name
        '         2009 Population Estimation
        '         2009 Births
        '         2009 Deaths
        '       Demographic information is from the US Census Bureau.
        sourceLines = Split(My.Resources.Counties, vbCrLf)
        For Each oneLine In sourceLines
            ' ----- Add a table row.
            lineFields = Split(oneLine, ",")
            oneRow = result.NewRow()
            oneRow!ParentID = lineFields(0)
            oneRow!CountyName = lineFields(1)
            oneRow!Population2009 = CInt(lineFields(2))
            oneRow!Births2009 = CInt(lineFields(3))
            oneRow!Deaths2009 = CInt(lineFields(4))
            result.Rows.Add(oneRow)
        Next oneLine

        ' ----- Finished.
        Return result
    End Function

    Private Sub ResetDataTables()
        ' ----- Reset the table data used by the sample form.
        Dim parentTable As DataTable
        Dim childTable As DataTable

        ' ----- Clear the existing data.
        ChildRecords.DataSource = Nothing
        ParentRecords.DataSource = Nothing
        ExampleData = New DataSet

        ' ----- Add the tables and relationships.
        parentTable = GetParentTable()
        childTable = GetChildTable()
        ExampleData.Tables.Add(parentTable)
        ExampleData.Tables.Add(childTable)
        ExampleData.Relations.Add(parentTable.Columns!ID,
            childTable.Columns!ParentID)

        ' ----- Add the tables to the display.
        ParentRecords.DataSource = parentTable
        ChildRecords.DataSource = childTable
    End Sub

    Private Sub RefreshComputeColumns()
        ' ----- Build a list of available columns from both parent and child.
        Dim scanColumn As DataColumn

        ' ----- Clear any existing definitions.
        ComputeColumn.Items.Clear()

        ' ----- Add the parent columns.
        For Each scanColumn In ExampleData.Tables("Parent").Columns
            ComputeColumn.Items.Add("Parent." & scanColumn.ColumnName)
        Next scanColumn

        ' ----- Add the child columns.
        For Each scanColumn In ExampleData.Tables("Child").Columns
            ComputeColumn.Items.Add("Child." & scanColumn.ColumnName)
        Next scanColumn

        ' ----- Finished.
        ComputeColumn.SelectedIndex = 0
    End Sub
End Class