﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 6 - Visual Basic Version
'       by Tim Patrick

Public Class DataViews
    Private SampleData As DataTable

    Private Sub DataViews_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim scanColumn As DataColumn

        ' ----- Add the sample data to the form.
        SampleData = GetOriginalTable()
        OriginalRecords.DataSource = SampleData

        ' ----- Show the list of columns.
        IncludedColumns.Items.Clear()
        For Each scanColumn In SampleData.Columns
            IncludedColumns.Items.Add(scanColumn.ColumnName)
        Next scanColumn
        IncludedColumns.SelectedIndex = 0
        RefreshColumnButtons()
    End Sub

    Private Sub ActExtract_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActExtract.Click
        ' ----- Create a new table based on the original.
        Dim interimView As DataView
        Dim generatedTable As DataTable = Nothing

        ' ----- At least one column is required.
        If (IncludedColumns.CheckedItems.Count = 0) Then
            MessageBox.Show("Please include at least one column.")
            IncludedColumns.Focus()
            Return
        End If

        ' ----- Clear the previous results.
        NewRecords.DataSource = Nothing

        ' ----- Build a view that will generate the new table.
        interimView = New DataView(SampleData)

        ' ----- Apply the optional filter.
        Try
            If (FilterExpression.Text.Trim.Length > 0) Then
                interimView.RowFilter = FilterExpression.Text.Trim
            End If
        Catch ex As Exception
            MessageBox.Show("Could not apply the filter: " & ex.Message)
            Return
        End Try

        ' ----- Generate the new table.
        Try
            generatedTable = interimView.ToTable(True,
                IncludedColumns.CheckedItems.Cast(Of String).ToArray())
        Catch ex As Exception
            MessageBox.Show("Could not build the new table: " & ex.Message)
            Return
        End Try

        ' ----- Display the results.
        NewRecords.DataSource = generatedTable
    End Sub

    Private Sub ActMoveUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActMoveUp.Click
        ' ----- Move the selected column up one position.
        Dim content As Object
        Dim selection As Boolean
        Dim startIndex As Integer

        ' ----- Identify the base item to move.
        startIndex = IncludedColumns.SelectedIndex
        If (startIndex <= 0) Then Return

        ' ----- Move the content.
        content = IncludedColumns.Items(startIndex)
        IncludedColumns.Items(startIndex) = IncludedColumns.Items(startIndex - 1)
        IncludedColumns.Items(startIndex - 1) = content

        ' ----- Move the inclusion marker.
        selection = IncludedColumns.GetItemChecked(startIndex)
        IncludedColumns.SetItemChecked(startIndex,
            IncludedColumns.GetItemChecked(startIndex - 1))
        IncludedColumns.SetItemChecked(startIndex - 1, selection)

        ' ----- Move to the new position.
        IncludedColumns.SelectedIndex = startIndex - 1
    End Sub

    Private Sub ActMoveDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActMoveDown.Click
        ' ----- Move the selected column down one position.
        Dim content As Object
        Dim selection As Boolean
        Dim startIndex As Integer

        ' ----- Identify the base item to move.
        startIndex = IncludedColumns.SelectedIndex
        If (startIndex = -1) Then Return
        If (startIndex = (IncludedColumns.Items.Count - 1)) Then Return

        ' ----- Move the content.
        content = IncludedColumns.Items(startIndex)
        IncludedColumns.Items(startIndex) = IncludedColumns.Items(startIndex + 1)
        IncludedColumns.Items(startIndex + 1) = content

        ' ----- Move the inclusion marker.
        selection = IncludedColumns.GetItemChecked(startIndex)
        IncludedColumns.SetItemChecked(startIndex,
            IncludedColumns.GetItemChecked(startIndex + 1))
        IncludedColumns.SetItemChecked(startIndex + 1, selection)

        ' ----- Move to the new position.
        IncludedColumns.SelectedIndex = startIndex + 1
    End Sub

    Private Sub IncludedColumns_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles IncludedColumns.SelectedIndexChanged
        ' ----- Highlight the right buttons.
        RefreshColumnButtons()
    End Sub

    Private Function GetOriginalTable() As DataTable
        ' ----- Build a sample original table.
        Dim sourceLines() As String
        Dim oneLine As String
        Dim lineFields() As String
        Dim oneRow As DataRow
        Dim result As New DataTable("Original")

        ' ----- Add the column definitions.
        result.Columns.Add("ID", GetType(String))
        result.Columns.Add("StateName", GetType(String))
        result.Columns.Add("AreaSqMiles", GetType(Integer))
        result.Columns.Add("Statehood", GetType(Date))
        result.Columns.Add("Capital", GetType(String))

        ' ----- Add some sample data. The "States" resource has the
        '       following comma-delimited fields:
        '         ID (a.k.a., State Abbreviation)
        '         State Name
        '         Area in Square Miles
        '         Date of Statehood (optional)
        '         Capital (optional)
        sourceLines = Split(My.Resources.States, vbCrLf)
        For Each oneLine In sourceLines
            ' ----- Add a table row.
            lineFields = Split(oneLine, ",")
            oneRow = result.NewRow()
            oneRow!ID = lineFields(0)
            oneRow!StateName = lineFields(1)
            oneRow!AreaSqMiles = CInt(lineFields(2))
            If (lineFields(3).Length > 0) Then oneRow!Statehood = CDate(lineFields(3))
            If (lineFields(4).Length > 0) Then oneRow!Capital = lineFields(4)
            result.Rows.Add(oneRow)
        Next oneLine

        ' ----- Finished.
        Return result
    End Function

    Private Sub RefreshColumnButtons()
        ' ----- Enable the movement buttons as needed.
        ActMoveUp.Enabled = (IncludedColumns.SelectedIndex > 0)
        ActMoveDown.Enabled = (IncludedColumns.SelectedIndex >= 0) _
            And (IncludedColumns.SelectedIndex < (IncludedColumns.Items.Count - 1))
    End Sub
End Class
