﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 9 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class StateBuilder
    Private ActiveStateID As Long = -1&

    Private Sub RefreshEverything()
        ' ----- Refresh most of the fields on the three main tabs.
        Dim sqlText As String
        Dim linkToDB As SqlConnection
        Dim stateReader As SqlDataReader

        ' ----- Initialize.
        ActiveStateID = -1&

        ' ----- Open a database connection.
        linkToDB = OpenConnection()
        If (linkToDB Is Nothing) Then Return

        ' ----- See if a custom state already exists.
        sqlText = "SELECT * FROM StateRegion WHERE RegionType = 99"
        stateReader = OpenReader(sqlText, linkToDB)
        If (stateReader IsNot Nothing) AndAlso (stateReader.HasRows = True) Then
            ' ----- Existing custom state record.
            stateReader.Read()
            ActiveStateID = CLng(stateReader!ID)

            AddName.Text = CStr(stateReader!FullName)
            AddAbbreviation.Text = CStr(stateReader!Abbreviation)
        Else
            ' ----- No custom state record.
            AddName.Clear()
            AddAbbreviation.Clear()
        End If
        If (stateReader IsNot Nothing) Then stateReader.Close()

        ' ----- Adjust the display.
        If (ActiveStateID = -1&) Then
            ' ----- Set up the display for a new entry.
            AddName.Enabled = True
            AddAbbreviation.Enabled = True
            ActAdd.Enabled = True

            EditID.Text = "N/A"
            EditName.Clear()
            EditAbbreviation.Clear()
            EditName.Enabled = False
            EditAbbreviation.Enabled = False
            ActEdit.Enabled = False

            DeleteID.Text = "N/A"
            DeleteName.Text = "N/A"
            DeleteAbbreviation.Text = "N/A"
            ActDelete.Enabled = False
        Else
            ' ----- Set up the display for an existing item.
            AddName.Enabled = False
            AddAbbreviation.Enabled = False
            ActAdd.Enabled = False

            EditID.Text = CStr(ActiveStateID)
            EditName.Text = AddName.Text
            EditAbbreviation.Text = AddAbbreviation.Text
            EditName.Enabled = True
            EditAbbreviation.Enabled = True
            ActEdit.Enabled = True

            DeleteID.Text = CStr(ActiveStateID)
            DeleteName.Text = AddName.Text
            DeleteAbbreviation.Text = AddAbbreviation.Text
            ActDelete.Enabled = True
        End If

        ' ----- Refresh the SQL statement previews.
        RefreshAddPreview()
        RefreshEditPreview()
        RefreshDeletePreview()
    End Sub

    Private Sub RefreshAddPreview()
        ' ----- Show the processing string.
        If (ActiveStateID = -1&) Then
            AddSQL.Text = GenerateAddSQL()
        Else
            AddSQL.Text = "Custom state already exists."
        End If
    End Sub

    Private Sub RefreshEditPreview()
        ' ----- Show the processing string.
        If (ActiveStateID = -1&) Then
            EditSQL.Text = "No custom state available."
        Else
            EditSQL.Text = GenerateEditSQL()
        End If
    End Sub

    Private Sub RefreshDeletePreview()
        ' ----- Show the processing string.
        If (ActiveStateID = -1&) Then
            DeleteSQL.Text = "No custom state available."
        Else
            DeleteSQL.Text = GenerateDeleteSQL()
        End If
    End Sub

    Private Function GenerateAddSQL() As String
        ' ----- Build the SQL string that will add a state record.
        If (ActiveStateID = -1&) Then
            Return "INSERT INTO StateRegion (FullName, Abbreviation, " &
                "RegionType) OUTPUT INSERTED.ID VALUES (" &
                DBText(AddName.Text.Trim) & ", " &
                DBText(AddAbbreviation.Text.Trim) & ", 99)"
        Else
            Return ""
        End If
    End Function

    Private Function GenerateEditSQL() As String
        ' ----- Build the SQL string that will modify a state record.
        If (ActiveStateID = -1&) Then
            Return ""
        Else
            Return "UPDATE StateRegion " &
                "SET FullName = " & DBText(EditName.Text.Trim) &
                ", Abbreviation = " & DBText(EditAbbreviation.Text.Trim) &
                " WHERE ID = " & ActiveStateID
        End If
    End Function

    Private Function GenerateDeleteSQL() As String
        ' ----- Build the SQL string that will delete a state record.
        If (ActiveStateID = -1&) Then
            Return ""
        Else
            Return "DELETE FROM StateRegion WHERE ID = " & ActiveStateID
        End If
    End Function

    Private Sub TextFields_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
            AddName.Enter, AddAbbreviation.Enter, EditName.Enter, EditAbbreviation.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub AddFields_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles _
            AddName.TextChanged, AddAbbreviation.TextChanged
        ' ----- Refresh the preview.
        AddSQL.Text = GenerateAddSQL()
    End Sub

    Private Sub EditFields_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles _
            EditName.TextChanged, EditAbbreviation.TextChanged
        ' ----- Refresh the preview.
        EditSQL.Text = GenerateEditSQL()
    End Sub

    Private Sub ActAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAdd.Click
        ' ----- Add the custom state.
        Dim linkToDB As SqlConnection
        Dim result As Object

        ' ----- Ignore if there is already a state.
        If (ActiveStateID <> -1&) Then Return

        ' ----- The state name is required.
        If (AddName.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a state name.")
            Return
        End If

        ' ----- The state abbreviaion is required.
        If (AddAbbreviation.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a state abbreviation.")
            Return
        End If

        ' ----- Access the database.
        linkToDB = OpenConnection()
        If (linkToDB Is Nothing) Then Return

        ' ----- Remove the record.
        result = ExecuteSQLReturn(GenerateAddSQL(), linkToDB)
        linkToDB.Close()
        If (IsDBNull(result) = True) Then Return
        MessageBox.Show("Added new state with ID " & CLng(result) & ".")

        ' ------ Refresh the display.
        RefreshEverything()
        ActionTabs.SelectedTab = TabEdit
    End Sub

    Private Sub ActEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActEdit.Click
        ' ----- Modify the custom state.
        Dim linkToDB As SqlConnection

        ' ----- Ignore if there is nothing to modify.
        If (ActiveStateID = -1&) Then Return

        ' ----- The state name is required.
        If (EditName.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a state name.")
            Return
        End If

        ' ----- The state abbreviaion is required.
        If (EditAbbreviation.Text.Trim.Length = 0) Then
            MessageBox.Show("Please provide a state abbreviation.")
            Return
        End If

        ' ----- Access the database.
        linkToDB = OpenConnection()
        If (linkToDB Is Nothing) Then Return

        ' ----- Modify the record.
        ExecuteSQL(GenerateEditSQL(), linkToDB)
        linkToDB.Close()
        MessageBox.Show("Modified the state record.")

        ' ------ Refresh the display.
        RefreshEverything()
    End Sub

    Private Sub ActDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDelete.Click
        ' ----- Delete the custom state.
        Dim linkToDB As SqlConnection

        ' ----- Ignore if there is nothing to delete.
        If (ActiveStateID = -1&) Then Return

        ' ----- Access the database.
        linkToDB = OpenConnection()
        If (linkToDB Is Nothing) Then Return

        ' ----- Remove the record.
        ExecuteSQL(GenerateDeleteSQL(), linkToDB)
        linkToDB.Close()
        MessageBox.Show("Deleted the state record.")

        ' ------ Refresh the display.
        RefreshEverything()
        ActionTabs.SelectedTab = TabAdd
    End Sub

    Private Sub StateBuilder_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        RefreshEverything()
        If (ActiveStateID <> -1&) Then ActionTabs.SelectedTab = TabEdit
    End Sub
End Class
