﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 10 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class ManageCustomers
    ' ----- Local information store of customer elements.
    Private Class CustomerInfo
        Public ID As Long
        Public Name As String
        Public AnnualFee As Decimal
        Public TotalOrders As Integer

        Public Overrides Function Equals(ByVal obj As Object) As Boolean
            ' ----- Locate a matching customer ID.
            If (TypeOf obj Is Long) Then
                If (Me.ID = CLng(obj)) Then Return True Else Return False
            Else
                Return MyBase.Equals(obj)
            End If
        End Function
    End Class

    Private Sub ManageCustomers_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Start with the list of customers.
        RefreshCustomerList()
    End Sub

    Private Sub AllCustomers_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles AllCustomers.DoubleClick
        ' ----- Same as the View Orders button.
        ActViewOrders.PerformClick()
    End Sub

    Private Sub AllCustomers_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles AllCustomers.DrawItem
        ' ----- Draw a single display item.
        Dim useBrush As System.Drawing.Brush
        Dim itemDetail As CustomerInfo
        Dim ellipsesFormat As StringFormat

        ' ----- Ignore undrawable items.
        If (e.Index = -1) Then Return

        ' ----- Draw the background of the item.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) _
            Then useBrush = SystemBrushes.HighlightText _
            Else useBrush = SystemBrushes.WindowText
        e.DrawBackground()

        ' ----- Extract the detail from the list.
        itemDetail = CType(AllCustomers.Items(e.Index), CustomerInfo)

        ' ----- Some data may not fit.
        ellipsesFormat = New StringFormat
        ellipsesFormat.Trimming = StringTrimming.EllipsisCharacter

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(itemDetail.Name, e.Font, useBrush,
            New Rectangle(0, e.Bounds.Top, ColAnnualFee.Left - ColCustomerName.Left - 5,
            e.Bounds.Height), ellipsesFormat)
        e.Graphics.DrawString(Format(itemDetail.AnnualFee, "Currency"),
            e.Font, useBrush, ColAnnualFee.Left, e.Bounds.Top)
        e.Graphics.DrawString(CStr(itemDetail.TotalOrders),
            e.Font, useBrush, ColOrders.Left, e.Bounds.Top)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
    End Sub

    Private Sub AllCustomers_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles AllCustomers.SelectedIndexChanged
        ' ----- Enable the controls as needed.
        ActRename.Enabled = (AllCustomers.SelectedIndex <> -1)
        ActAnnualFee.Enabled = (AllCustomers.SelectedIndex <> -1)
        ActViewOrders.Enabled = (AllCustomers.SelectedIndex <> -1)
    End Sub

    Private Sub ActRename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActRename.Click
        ' ----- Rename the selected customer.
        Dim whichCustomer As CustomerInfo

        ' ----- Access the customer record.
        If (AllCustomers.SelectedIndex = -1) Then
            MessageBox.Show("Please select a customer from the list.")
            Return
        End If
        whichCustomer = CType(AllCustomers.SelectedItem, CustomerInfo)

        ' ----- Prompt to rename the customer.
        If ((New RenameCustomer).PromptUser(whichCustomer.ID) = True) Then
            ' ----- Refresh the list to show the new name.
            RefreshCustomerList()
            AllCustomers.SelectedIndex = AllCustomers.Items.IndexOf(whichCustomer.ID)
        End If
    End Sub

    Private Sub ActAnnualFee_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAnnualFee.Click
        ' ----- Change the annual fee of the selected customer.
        Dim whichCustomer As CustomerInfo

        ' ----- Access the customer record.
        If (AllCustomers.SelectedIndex = -1) Then
            MessageBox.Show("Please select a customer from the list.")
            Return
        End If
        whichCustomer = CType(AllCustomers.SelectedItem, CustomerInfo)

        ' ----- Prompt to rename the customer.
        If ((New ChangeAnnualFee).PromptUser(whichCustomer.ID) = True) Then
            ' ----- Refresh the list to show the annual fee.
            RefreshCustomerList()
            AllCustomers.SelectedIndex = AllCustomers.Items.IndexOf(whichCustomer.ID)
        End If
    End Sub

    Private Sub ActViewOrders_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActViewOrders.Click
        ' ----- Display the orders of the selected customer.
        Dim whichCustomer As CustomerInfo

        ' ----- Access the customer record.
        If (AllCustomers.SelectedIndex = -1) Then
            MessageBox.Show("Please select a customer from the list.")
            Return
        End If
        whichCustomer = CType(AllCustomers.SelectedItem, CustomerInfo)

        ' ----- Prompt to rename the customer.
        Call (New ViewOrders).ShowOrders(whichCustomer.ID)
    End Sub

    Private Sub RefreshCustomerList()
        ' ----- Load in the list of customers.
        Dim sqlText As String
        Dim customerReader As SqlDataReader = Nothing
        Dim linkToDB As SqlConnection = Nothing
        Dim oneCustomer As CustomerInfo

        ' ----- Clear any previous customers.
        AllCustomers.Items.Clear()

        ' ----- Retrieve the customers.
        sqlText = "SELECT CU.ID, CU.FullName, CU.AnnualFee, COUNT(OE.ID) AS Orders " & _
            "FROM Customer AS CU LEFT JOIN OrderEntry AS OE ON " & _
            "CU.ID = OE.Customer GROUP BY CU.ID, CU.FullName, CU.AnnualFee " & _
            "ORDER BY CU.FullName"
        Try
            ' ----- Access the database.
            linkToDB = OpenConnection()
            If (linkToDB Is Nothing) Then Return

            ' ----- Read in the records.
            customerReader = OpenReader(sqlText, linkToDB)
            Do While (customerReader.Read = True)
                ' ----- Add a new customer entry.
                oneCustomer = New CustomerInfo
                oneCustomer.ID = CLng(customerReader!ID)
                oneCustomer.Name = CStr(customerReader!FullName)
                oneCustomer.AnnualFee = CDec(customerReader!AnnualFee)
                oneCustomer.TotalOrders = CInt(customerReader!Orders)
                AllCustomers.Items.Add(oneCustomer)
            Loop
        Catch ex As Exception
            MessageBox.Show("Error while accessing customer list: " & ex.Message)
        Finally
            If (customerReader IsNot Nothing) Then customerReader.Close()
            If (linkToDB IsNot Nothing) Then linkToDB.Dispose()
        End Try
    End Sub
End Class
