﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 16 - Visual Basic Version
'       by Tim Patrick

Public Class CustomerDetail
    Private ActiveCustomer As Customer
    Private ActiveContext As SalesOrderEntities

    Public Function EditCustomer(ByRef oneCustomer As Customer,
            ByVal context As SalesOrderEntities) As Boolean
        ' ----- Prompt the user to edit a new or existing customer.
        Me.ActiveCustomer = oneCustomer
        Me.ActiveContext = context
        If (Me.ShowDialog() = DialogResult.OK) Then
            If (oneCustomer Is Nothing) Then oneCustomer = Me.ActiveCustomer
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub TextFields_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles _
            CustomerName.Enter, Address1.Enter, Address2.Enter, CityName.Enter,
            PostalCode.Enter, PhoneNumber.Enter, WebSite.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub CustomerDetail_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Load in the list of states/regions.
        Dim scanState As StateRegion

        ' ----- Add each state to the state drop-down.
        StateName.Items.Add(New ItemData("<Not Selected>", -1&))
        For Each scanState In Me.ActiveContext.StateRegions.OrderBy("it.FullName")
            StateName.Items.Add(New ItemData(scanState.FullName, scanState.ID))
        Next scanState
        StateName.SelectedIndex = 0

        ' ----- Fill in the existing details.
        If (Me.ActiveCustomer IsNot Nothing) Then
            CustomerName.Text = Me.ActiveCustomer.FullName
            If (Me.ActiveCustomer.Address1 IsNot Nothing) Then _
                Address1.Text = Me.ActiveCustomer.Address1
            If (Me.ActiveCustomer.Address2 IsNot Nothing) Then _
                Address2.Text = Me.ActiveCustomer.Address2
            If (Me.ActiveCustomer.City IsNot Nothing) Then _
                CityName.Text = Me.ActiveCustomer.City
            If (Me.ActiveCustomer.StateRegion IsNot Nothing) Then _
                StateName.SelectedIndex = StateName.Items.IndexOf(Me.ActiveCustomer.StateRegion)
            If (Me.ActiveCustomer.PostalCode IsNot Nothing) Then _
                PostalCode.Text = Me.ActiveCustomer.PostalCode
            If (Me.ActiveCustomer.PhoneNumber IsNot Nothing) Then _
                PhoneNumber.Text = Me.ActiveCustomer.PhoneNumber
            If (Me.ActiveCustomer.WebSite IsNot Nothing) Then _
                WebSite.Text = Me.ActiveCustomer.WebSite
            AnnualFee.Value = Me.ActiveCustomer.AnnualFee
        End If
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Update the customer instance with the edited values.
        Dim toUpdate As Customer

        If (Me.ActiveCustomer Is Nothing) Then
            ' ----- New customer.
            toUpdate = New Customer
        Else
            ' ----- Existing customer.
            toUpdate = Me.ActiveCustomer
        End If

        ' ----- Update the individual fields.

        ' ----- Update the database.
        Try

        Catch ex As Exception
            MessageBox.Show("Error saving the customer record: " & ex.Message)
            Return False
        End Try
    End Function

    Private Function VerifyFormData() As Boolean
        ' ----- A customer name is required.
        If (CustomerName.Text.Trim.Length = 0) Then
            MessageBox.Show("A customer name is required.")
            CustomerName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Confirm and save the data.
        If (VerifyFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub
End Class
