﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 3 - Visual Basic Version
'       by Tim Patrick

Public Class AccountDetail
    Private AccountEntry As DataRow
    Private AccountTable As DataTable

    Public Function EditAccount(ByRef whichAccount As DataRow,
            ByVal baseTable As DataTable) As Boolean
        ' ----- Prompt the user to edit the account.
        AccountEntry = whichAccount
        AccountTable = baseTable
        If (Me.ShowDialog() = DialogResult.OK) Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub AccountDetail_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.

    End Sub

    Private Sub TextFields_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) _
            Handles AccountName.Enter, AnnualFee.Enter, StartDate.Enter
        ' ----- Highlight the entire field.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Perform some basic data checks.
        Dim workArea As DataRow

        ' ----- Annual Fee, if supplied, must be numeric.
        If (AnnualFee.Text.Trim.Length > 0) AndAlso
                (IsNumeric(AnnualFee.Text) = False) Then
            MessageBox.Show("Annual Fee, if supplied, must be numeric.")
            Return
        End If

        ' ----- Start Date, if supplied, must be a valid date.
        If (StartDate.Text.Trim.Length > 0) AndAlso
                (IsDate(StartDate.Text) = False) Then
            MessageBox.Show("Start Date, if supplied, must be a valid date.")
            Return
        End If

        ' ----- Obtain the record to update.
        workArea = AccountEntry
        If (workArea Is Nothing) Then workArea = AccountTable.NewRow()
        workArea.ClearErrors()

        Try
            ' ----- Save the changes in the record.
            workArea.BeginEdit()

            ' ----- Check for column-level errors.
            If (workArea.HasErrors()) Then
                ReportRowError(workArea)
                workArea.CancelEdit()
                Return
            End If

            ' ----- Check for row-level errors.
            workArea.EndEdit()
            If (workArea.HasErrors()) Then
                ReportRowError(workArea)
                Return
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Return
        End Try

        ' ----- Success.
        AccountEntry = workArea
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub ReportRowError(ByVal whichRow As DataRow,
            Optional ByVal messagePrefix As String = "")
        ' ----- Show first column-level or row-level error.
        Dim errorText As String = ""
        Dim errorColumns() As DataColumn = whichRow.GetColumnsInError()

        If (errorColumns.Count > 0) Then
            errorText = whichRow.GetColumnError(errorColumns(0))
        ElseIf (whichRow.RowError.Length > 0) Then
            errorText = whichRow.RowError
        End If
        If (errorText.Length = 0) Then errorText = "An unknown error occurred."
        MessageBox.Show(messagePrefix & errorText)
    End Sub
End Class
