﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 4 - Visual Basic Version
'       by Tim Patrick

Public Class TableExaminer

    Private Sub TableExaminer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim scanType As String
        Dim supportedTypes() As String =
            {"Boolean", "Byte", "Char", "DateTime", "Decimal", "Double",
            "Int16", "Int32", "Int64", "SByte", "Single", "String",
            "TimeSpan", "UInt16", "UInt32", "UInt64"}

        ' ----- Display the sample data.
        BaseTableDisplay.DataSource = BuildSampleTable()

        ' ----- Fill in the list of data types for the custom columns.
        For Each scanType In supportedTypes
            ColumnType1.Items.Add(scanType)
            ColumnType2.Items.Add(scanType)
            ColumnType3.Items.Add(scanType)
        Next scanType
        ColumnType1.SelectedIndex = 0
        ColumnType2.SelectedIndex = 0
        ColumnType3.SelectedIndex = 0
    End Sub

    Private Function BuildSampleTable() As DataTable
        ' ------ Create a sample table to display in the application.
        Dim theTable As New DataTable("SampleTable")
        theTable.Columns.Add("ID", GetType(Long))
        theTable.Columns.Add("StudentName", GetType(String))
        theTable.Columns.Add("GradeYear", GetType(Integer))
        theTable.Columns.Add("ScoreTrimester1", GetType(Decimal))
        theTable.Columns.Add("ScoreTrimester2", GetType(Decimal))
        theTable.Columns.Add("ScoreTrimester3", GetType(Decimal))

        ' ----- Create a primary key for the table.
        theTable.PrimaryKey = {theTable.Columns("ID")}

        theTable.Rows.Add({3429&, "Abercrombie, Kim", 12, 4@, 3.9@, 4@})
        theTable.Rows.Add({2352&, "Hamlin, Jay", 10, 2.5@, 1.85@, 1.7@})
        theTable.Rows.Add({6843&, "Jacobsen, Lola", 10, 2.67@, 2.9@, 3.2@})
        theTable.Rows.Add({4810&, "Price, Jeff", 11, 3@, 3@, 3.4@})

        Return theTable
    End Function

    Private Sub TextFields_Enter(ByVal sender As Object, ByVal e As System.EventArgs) _
            Handles PrimaryKey.Enter, CriteriaFilter.Enter, CriteriaSorting.Enter,
            ColumnName1.Enter, ColumnName2.Enter, ColumnName3.Enter,
            ColumnExpression1.Enter, ColumnExpression2.Enter, ColumnExpression3.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub ActPrimaryKey_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActPrimaryKey.Click
        ' ----- Perform a primary key lookup.
        Dim usePrimaryKey As Long
        Dim result As DataRow = Nothing
        Dim workTable As DataTable
        Dim displayTable As DataTable

        ' ----- The primary key is required, and must be an integral value.
        If (IsNumeric(PrimaryKey.Text.Trim) = False) Or
                (InStr(PrimaryKey.Text, ".") > 0) Then
            MessageBox.Show("The Primary Key must be a valid integer.")
            PrimaryKey.Focus()
            Return
        End If

        ' ----- Convert supplied key to Long, checking for overflow.
        Try
            usePrimaryKey = CLng(PrimaryKey.Text)
        Catch ex As Exception
            MessageBox.Show("The Primary Key must be a valid integer.")
            PrimaryKey.Focus()
            Return
        End Try

        ' ----- Get a copy of the table for the lookup.
        workTable = BuildSampleTable()

        ' ----- Perform the lookup.
        Try

        Catch ex As MissingPrimaryKeyException
            MessageBox.Show("The table does not have a defined primary key.")
            Return
        Catch ex As Exception
            MessageBox.Show("Primary key lookup failed with the following error: " &
                ex.Message)
            Return
        End Try

        ' ----- Check for no match.
        If (result Is Nothing) Then
            MessageBox.Show("The supplied key matched no rows.")
            PrimaryKey.Focus()
            Return
        End If

        ' ----- Display the results. The results panel requires a DataTable,
        '       so build a new table that includes only the matching row.
        displayTable = workTable.Clone
        displayTable.ImportRow(result)
        Call (New ResultsViewer).ShowResults(displayTable)
    End Sub

    Private Sub ActCriteria_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActCriteria.Click
        ' ----- Perform a criteria lookup, with sorting.
        Dim workTable As DataTable
        Dim displayTable As DataTable
        Dim results() As DataRow = Nothing
        Dim scanResult As DataRow

        ' ----- Either a filter or a sorting list is required.
        If (CriteriaFilter.Text.Trim.Length = 0) And _
                (CriteriaSorting.Text.Trim.Length = 0) Then
            MessageBox.Show("You must provide either a filter or a sorting list.")
            Return
        End If

        ' ----- Build a local table to perform the filter/sort.
        workTable = BuildSampleTable()
        workTable.CaseSensitive = BeCaseSensitive.Checked

        ' ----- Apply the filter and sorting list.
        Try

        Catch ex As Exception
            MessageBox.Show("The filter failed with the following error: " &
                ex.Message)
            Return
        End Try

        ' ----- Display the results. The results panel requires a DataTable,
        '       so build a new table that includes only the matching rows.
        displayTable = workTable.Clone
        For Each scanResult In results
            displayTable.ImportRow(scanResult)
        Next scanResult
        Call (New ResultsViewer).ShowResults(displayTable)
    End Sub

    Private Sub ActExpression_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActExpression.Click
        ' ----- Add expression columns to the sample table.
        Dim workTable As DataTable
        Dim counter As Integer
        Dim numAdded As Integer
        Dim nameField As TextBox
        Dim typeField As ComboBox
        Dim expressionField As TextBox

        ' ----- Build a table for the new columns.
        workTable = BuildSampleTable()
        workTable.CaseSensitive = BeCaseSensitive.Checked

        ' ----- Scan each of the three expression columns.
        numAdded = 0
        For counter = 1 To 3
            ' ----- Locate the correct controls.
            nameField = CType(ByExpressionColumns.Controls("ColumnName" &
                counter), TextBox)
            typeField = CType(ByExpressionColumns.Controls("ColumnType" &
                counter), ComboBox)
            expressionField = CType(ByExpressionColumns.Controls("ColumnExpression" &
                counter), TextBox)

            ' ----- Ignore unused columns.
            If (nameField.Text.Trim.Length = 0) And _
                (expressionField.Text.Trim.Length = 0) Then Continue For

            ' ----- Warn on missing data.
            If (nameField.Text.Trim.Length = 0) Then
                MessageBox.Show("A name is missing for column #" & counter & ".")
                nameField.Focus()
                Return
            End If
            If (expressionField.Text.Trim.Length = 0) Then
                MessageBox.Show("An expression is missing for column #" & counter & ".")
                expressionField.Focus()
                Return
            End If

            ' ----- Add the expression column.
            Try

            Catch ex As Exception
                MessageBox.Show("Error occurred when adding expression column " &
                    counter & ": " & ex.Message)
                Return
            End Try
            numAdded += 1
        Next counter

        ' ----- If no columns added, don't bother.
        If (numAdded = 0) Then
            MessageBox.Show("You must add at least one expression column")
            Return
        End If

        ' ----- Show the results.
        Call (New ResultsViewer).ShowResults(workTable)
    End Sub
End Class
