﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 5 - Visual Basic Version
'       by Tim Patrick

Public Class FlightInfo
    Private SampleDataSet As DataSet

    Private Sub FlightInfo_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Fill in the update and delete rule lists.
        UpdateRule.Items.Add(Data.Rule.None)
        UpdateRule.Items.Add(Data.Rule.Cascade)
        UpdateRule.Items.Add(Data.Rule.SetNull)
        UpdateRule.Items.Add(Data.Rule.SetDefault)
        UpdateRule.SelectedIndex = 0

        DeleteRule.Items.Add(Data.Rule.None)
        DeleteRule.Items.Add(Data.Rule.Cascade)
        DeleteRule.Items.Add(Data.Rule.SetNull)
        DeleteRule.Items.Add(Data.Rule.SetDefault)
        DeleteRule.SelectedIndex = 0

        ' ----- Create the sample data.
        ActResetAll.PerformClick()
    End Sub

    Private Function BuildFlightTable() As DataTable
        ' ----- Create a sample table of flights.
        Dim result As DataTable
        Dim keyColumn As DataColumn

        ' ----- Design the table structure.
        result = New DataTable("Flight")
        keyColumn = result.Columns.Add("ID", GetType(Integer))
        result.Columns.Add("Region", GetType(String))
        result.Columns.Add("FlightsPerWeek", GetType(Integer))
        result.PrimaryKey = {keyColumn}

        ' ----- Add some sample data.
        result.Rows.Add({834, "Northwest", 5})
        result.Rows.Add({223, "South", 3})
        result.Rows.Add({225, "South", 5})

        Return result
    End Function

    Private Function BuildLegTable() As DataTable
        ' ----- Create a sample table of flight legs.
        Dim result As DataTable
        Dim keyColumn As DataColumn

        ' ----- Design the table structure.
        result = New DataTable("Leg")
        keyColumn = result.Columns.Add("ID", GetType(Integer))
        keyColumn.AutoIncrement = True
        result.Columns.Add("FlightID", GetType(Integer))
        result.Columns.Add("StartCity", GetType(String))
        result.Columns.Add("EndCity", GetType(String))
        result.Columns.Add("Distance", GetType(Integer))
        result.PrimaryKey = {keyColumn}

        ' ----- Add some sample data.
        result.Rows.Add({1, 834, "Portland (PDX)", "Seattle (SEA)", 129})
        result.Rows.Add({2, 834, "Seattle (SEA)", "Missoula (MSO)", 416})
        result.Rows.Add({3, 223, "Dallas (DFW)", "Albuquerque (ABQ)", 567})
        result.Rows.Add({4, 223, "Albuquerque (ABQ)", "Phoenix (PHX)", 328})
        result.Rows.Add({5, 223, "Phoenix (PHX)", "Los Angeles (LAX)", 369})
        result.Rows.Add({6, 225, "Los Angeles (LAX)", "Phoenix (PHX)", 369})
        result.Rows.Add({7, 225, "Phoenix (PHX)", "Albuquerque (ABQ)", 328})
        result.Rows.Add({8, 225, "Albuquerque (ABQ)", "Dallas (DFW)", 567})

        Return result
    End Function

    Private Function BuildSampleDataSet() As DataSet
        ' ----- Build a sample data set of flight information.
        Dim result As DataSet
        Dim parentTable As DataTable
        Dim childTable As DataTable
        Dim tableLink As DataRelation

        ' ----- Add the two tables to the data set.

        ' ----- Build the relationship between the tables.

        ' ----- Set the constraints as selected by the user.
        RefreshConstraints()
        result.EnforceConstraints = EnforceConstraints.Checked

        Return result
    End Function

    Private Sub RefreshConstraints()
        ' ----- Set the delete and update rules for the table relationship.
        Dim linkConstraint As ForeignKeyConstraint

        ' ----- Ignore if there is no constraint.
        If (SampleDataSet Is Nothing) Then Return
        If (SampleDataSet.Relations.Count = 0) Then Return
        linkConstraint = SampleDataSet.Relations(0).ChildKeyConstraint
        If (linkConstraint Is Nothing) Then Return

        ' ----- Alter its cascade rules.

    End Sub

    Private Sub ActResetAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActResetAll.Click
        ' ----- Clear any previous data.
        FlightView.DataSource = Nothing
        LegView.DataSource = Nothing
        ActFlightEdit.Enabled = False
        ActFlightDelete.Enabled = False
        ActLegEdit.Enabled = False
        ActLegDelete.Enabled = False

        ' ----- Start a fresh data set for testing.
        SampleDataSet = BuildSampleDataSet()
        RefreshConstraints()
        FlightView.DataSource = SampleDataSet.Tables("Flight")
        LegView.DataSource = SampleDataSet.Tables("Leg")
    End Sub

    Private Sub EnforceConstraints_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EnforceConstraints.CheckedChanged
        ' ----- Toggle constraint enforcement.
        If (SampleDataSet IsNot Nothing) Then _
            SampleDataSet.EnforceConstraints = EnforceConstraints.Checked
    End Sub

    Private Sub FlightView_SelectionChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FlightView.SelectionChanged
        ' ----- Enable buttons if needed.
        ActFlightEdit.Enabled = (FlightView.SelectedRows.Count > 0)
        ActFlightDelete.Enabled = (FlightView.SelectedRows.Count > 0)
    End Sub

    Private Sub LegView_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles LegView.SelectionChanged
        ' ----- Enable buttons if needed.
        ActLegEdit.Enabled = (LegView.SelectedRows.Count > 0)
        ActLegDelete.Enabled = (LegView.SelectedRows.Count > 0)
    End Sub

    Private Sub UpdateRule_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UpdateRule.SelectedIndexChanged
        ' ----- Set the constraints as selected by the user.
        RefreshConstraints()
    End Sub

    Private Sub DeleteRule_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DeleteRule.SelectedIndexChanged
        ' ----- Set the constraints as selected by the user.
        RefreshConstraints()
    End Sub

    Private Sub ActFlightAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActFlightAdd.Click
        ' ----- Add a new flight record.
        Dim flightTable As DataTable
        Dim flightRow As DataRow

        ' ----- Create a new placeholder.
        flightTable = SampleDataSet.Tables("Flight")
        If (flightTable Is Nothing) Then Return
        flightRow = flightTable.NewRow()

        ' ----- Prompt the user.
        If ((New FlightDetail).EditDetail(flightRow) = False) Then Return

        ' ----- Add the row to the table.
        Try
            flightTable.Rows.Add(flightRow)
            flightTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not add new row: " & ex.Message)
        End Try
    End Sub

    Private Sub ActFlightEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActFlightEdit.Click
        ' ----- Edit an existing flight record.
        Dim flightTable As DataTable
        Dim flightRowManaged As DataRowView
        Dim flightRow As DataRow

        ' ----- Locate the existing row.
        flightTable = SampleDataSet.Tables("Flight")
        If (flightTable Is Nothing) Then Return
        If (FlightView.SelectedRows.Count = 0) Then Return
        flightRowManaged = CType(FlightView.SelectedRows(0).DataBoundItem, DataRowView)
        flightRow = flightRowManaged.Row

        ' ----- Prompt the user.
        If ((New FlightDetail).EditDetail(flightRow) = False) Then Return

        ' ----- Update the row in the table.
        Try
            flightTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not edit the row: " & ex.Message)
        End Try
    End Sub

    Private Sub ActFlightDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActFlightDelete.Click
        ' ----- Add a new flight record.
        Dim flightTable As DataTable
        Dim flightRowManaged As DataRowView
        Dim flightRow As DataRow

        ' ----- Locate the existing row.
        flightTable = SampleDataSet.Tables("Flight")
        If (flightTable Is Nothing) Then Return
        If (FlightView.SelectedRows.Count = 0) Then Return
        flightRowManaged = CType(FlightView.SelectedRows(0).DataBoundItem, DataRowView)
        flightRow = flightRowManaged.Row

        ' ----- Prompt the user.
        If (MessageBox.Show("Really delete the selected flight?",
            "Chapter 5", MessageBoxButtons.YesNo, MessageBoxIcon.Question,
            MessageBoxDefaultButton.Button2) <> Windows.Forms.DialogResult.Yes) _
            Then Return

        ' ----- Update the row in the table.
        Try
            flightRow.Delete()
            flightTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not delete the row: " & ex.Message)
        End Try
    End Sub

    Private Sub ActLegAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActLegAdd.Click
        ' ----- Add a new leg record.
        Dim legTable As DataTable
        Dim legRow As DataRow
        Dim flightRowManaged As DataRowView
        Dim flightRow As DataRow
        Dim flightID As String

        ' ----- Create a new placeholder.
        legTable = SampleDataSet.Tables("Leg")
        If (legTable Is Nothing) Then Return
        legRow = legTable.NewRow()

        ' ----- Get the selected flight ID.
        flightID = ""
        If (FlightView.SelectedRows.Count > 0) Then
            flightRowManaged = CType(FlightView.SelectedRows(0).DataBoundItem, DataRowView)
            flightRow = flightRowManaged.Row
            flightID = CStr(flightRow!ID)
        End If

        ' ----- Prompt the user.
        If ((New LegDetail).EditDetail(legRow, flightID) = False) Then Return

        ' ----- Add the row to the table.
        Try
            legTable.Rows.Add(legRow)
            legTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not add new row: " & ex.Message)
        End Try
    End Sub

    Private Sub ActLegEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActLegEdit.Click
        ' ----- Edit an existing leg record.
        Dim legTable As DataTable
        Dim legRowManaged As DataRowView
        Dim legRow As DataRow
        Dim flightRowManaged As DataRowView
        Dim flightRow As DataRow
        Dim flightID As String

        ' ----- Locate the existing row.
        legTable = SampleDataSet.Tables("Leg")
        If (legTable Is Nothing) Then Return
        If (LegView.SelectedRows.Count = 0) Then Return
        legRowManaged = CType(LegView.SelectedRows(0).DataBoundItem, DataRowView)
        legRow = legRowManaged.Row

        ' ----- Get the selected flight ID.
        flightID = ""
        If (FlightView.SelectedRows.Count > 0) Then
            flightRowManaged = CType(FlightView.SelectedRows(0).DataBoundItem, DataRowView)
            flightRow = flightRowManaged.Row
            flightID = CStr(flightRow!ID)
        End If

        ' ----- Prompt the user.
        If ((New LegDetail).EditDetail(legRow, flightID) = False) Then Return

        ' ----- Update the row in the table.
        Try
            legTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not edit the row: " & ex.Message)
        End Try
    End Sub

    Private Sub ActLegDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActLegDelete.Click
        ' ----- Add a new leg record.
        Dim legTable As DataTable
        Dim legRowManaged As DataRowView
        Dim legRow As DataRow

        ' ----- Locate the existing row.
        legTable = SampleDataSet.Tables("Leg")
        If (legTable Is Nothing) Then Return
        If (LegView.SelectedRows.Count = 0) Then Return
        legRowManaged = CType(LegView.SelectedRows(0).DataBoundItem, DataRowView)
        legRow = legRowManaged.Row

        ' ----- Prompt the user.
        If (MessageBox.Show("Really delete the selected leg?",
            "Chapter 5", MessageBoxButtons.YesNo, MessageBoxIcon.Question,
            MessageBoxDefaultButton.Button2) <> Windows.Forms.DialogResult.Yes) _
            Then Return

        ' ----- Update the row in the table.
        Try
            legRow.Delete()
            legTable.AcceptChanges()
        Catch ex As Exception
            MessageBox.Show("Could not delete the row: " & ex.Message)
        End Try
    End Sub
End Class
