﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 8 - Visual Basic Version
'       by Tim Patrick

Imports System.Data.SqlClient

Public Class ConnectionTest
    ' ----- Class to track text/value pairs in ComboBox controls.
    Private Class ItemData
        Public ItemText As String
        Public ItemValue As Integer

        Public Sub New(ByVal initialText As String, ByVal initialValue As Integer)
            Me.ItemText = initialText
            Me.ItemValue = initialValue
        End Sub

        Public Overrides Function ToString() As String
            Return ItemText
        End Function
    End Class

    Private Function BuildConnection() As SqlConnectionStringBuilder
        ' ----- Build a connection string based on user settings.
        Dim connection As New SqlConnectionStringBuilder

        Try
            ' ----- Add the server name.

            ' ----- Add the authentication.

            ' ----- Add the other settings.
            connection.InitialCatalog = InitialCatalog.Text
            connection.ConnectTimeout = CType(ConnectionTimeout.SelectedItem, 
                ItemData).ItemValue
            connection.MultipleActiveResultSets = EnableMarsSupport.Checked

            ' ----- Success.
            Return connection
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Return Nothing
        End Try
    End Function

    Private Sub RefreshConnectionPreview()
        ' ----- Refresh the preview of the connection string.
        Dim connection As SqlConnectionStringBuilder

        Try
            ' ----- Build the connection string from settings.
            connection = BuildConnection()
            If (connection Is Nothing) Then
                ConnectionPreview.Text = "!! Error"
            Else
                ConnectionPreview.Text = connection.ConnectionString
            End If
        Catch ex As Exception
            ConnectionPreview.Text = "!! Error"
        End Try
    End Sub

    Private Sub RemoteServer_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RemoteServer.CheckedChanged
        ' ----- Enable fields as needed.
        LabelServerName.Enabled = RemoteServer.Checked
        ServerName.Enabled = RemoteServer.Checked
        IsExpressEdition.Enabled = RemoteServer.Checked

        ' ----- Update the preview.
        RefreshConnectionPreview()
    End Sub

    Private Sub AuthenticateSqlServer_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AuthenticateSqlServer.CheckedChanged
        ' ----- Enable fields as needed.
        LabelUserName.Enabled = AuthenticateSqlServer.Checked
        UserName.Enabled = AuthenticateSqlServer.Checked
        LabelPassword.Enabled = AuthenticateSqlServer.Checked
        UserPassword.Enabled = AuthenticateSqlServer.Checked

        ' ----- Update the preview.
        RefreshConnectionPreview()
    End Sub

    Private Sub TextFields_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
            ServerName.Enter, UserName.Enter, UserPassword.Enter, InitialCatalog.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub FieldUpdates(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
            IsExpressEdition.CheckedChanged, ServerName.TextChanged, UserName.TextChanged,
            UserPassword.TextChanged, InitialCatalog.TextChanged, EnableMarsSupport.CheckedChanged,
            ConnectionTimeout.SelectedIndexChanged
        ' ----- Update the preview.
        RefreshConnectionPreview()
    End Sub

    Private Sub ActClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActClose.Click
        ' ----- Exit the program.
        Me.Close()
    End Sub

    Private Sub ActTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActTest.Click
        ' ----- See if the specified connection works.
        Dim connection As New SqlConnectionStringBuilder
        Dim testLink As SqlConnection = Nothing

        Try
            ' ----- Retrieve the connection.
            connection = BuildConnection()
            If (connection Is Nothing) Then Return

            ' ----- Test the connection.

            If (testLink.State = ConnectionState.Open) Then
                MessageBox.Show("Connection opened successfully.")
            Else
                MessageBox.Show("Connection failed to open.")
            End If
        Catch ex As Exception
            MessageBox.Show("Connection failed with the following error: " &
                ex.Message)
        Finally
            If (testLink IsNot Nothing) Then testLink.Dispose()
        End Try
    End Sub

    Private Sub ConnectionTest_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Fill in the list of timeout choices.
        ConnectionTimeout.Items.Add(New ItemData("15 Seconds", 15))
        ConnectionTimeout.Items.Add(New ItemData("30 Seconds", 30))
        ConnectionTimeout.Items.Add(New ItemData("45 Seconds", 45))
        ConnectionTimeout.Items.Add(New ItemData("60 Seconds", 60))
        ConnectionTimeout.SelectedIndex = 0

        ' ----- Show the initial connection string.
        RefreshConnectionPreview()
    End Sub
End Class
